@@ -50,13 +50,12 @@ interfaces_setup_multicast(struct lldpd *cfg, const char *name,
 
 	for (i = 0; cfg->g_protocols[i].mode != 0; i++) {
 		if (!cfg->g_protocols[i].enabled) continue;
-		for (mac = cfg->g_protocols[i].mac1, j = 0;
-		     j < 3;
-		     mac += ETHER_ADDR_LEN,
+		for (j = 0;
+		     j < sizeof(cfg->g_protocols[0].mac)/sizeof(cfg->g_protocols[0].mac[0]);
 		     j++) {
+			mac = cfg->g_protocols[i].mac[j];
 			if (memcmp(mac, zero, ETHER_ADDR_LEN) == 0) break;
-			if ((rc = priv_iface_multicast(name,
-				    mac, !remove)) != 0) {
+			if ((rc = priv_iface_multicast(name, mac, !remove)) != 0) {
 				errno = rc;
 				if (errno != ENOENT)
 					log_debug("interfaces",
@@ -588,7 +587,9 @@ interfaces_helper_port_name_desc(struct lldpd *cfg,
 	/* We need to set the portid to what the client configured.
 	   This can be done from the CLI.
 	*/
-	int has_alias = (iface->alias != NULL && strlen(iface->alias) != 0);
+	int has_alias = (iface->alias != NULL
+	    && strlen(iface->alias) != 0
+	    && strncmp("lldpd: ", iface->alias, 7));
 	int portid_type = cfg->g_config.c_lldp_portid_type;
 	if (portid_type == LLDP_PORTID_SUBTYPE_IFNAME ||
 	    (portid_type == LLDP_PORTID_SUBTYPE_UNKNOWN && has_alias) ||
