@@ -106,21 +106,23 @@ char *
 priv_gethostname()
 {
 	static char *buf = NULL;
-	int rc;
+	int len;
 	enum priv_cmd cmd = PRIV_GET_HOSTNAME;
 	must_write(PRIV_UNPRIVILEGED, &cmd, sizeof(enum priv_cmd));
 	priv_wait();
-	must_read(PRIV_UNPRIVILEGED, &rc, sizeof(int));
-	if ((buf = (char*)realloc(buf, rc+1)) == NULL)
+	must_read(PRIV_UNPRIVILEGED, &len, sizeof(int));
+	if (len < 0 || len > 255)
+		fatalx("privsep", "too large value requested");
+	if ((buf = (char*)realloc(buf, len+1)) == NULL)
 		fatal("privsep", NULL);
-	must_read(PRIV_UNPRIVILEGED, buf, rc);
-	buf[rc] = '\0';
+	must_read(PRIV_UNPRIVILEGED, buf, len);
+	buf[len] = '\0';
 	return buf;
 }
 
 
 int
-priv_iface_init(int index, char *iface, int proto)
+priv_iface_init(int index, char *iface)
 {
 	int rc;
 	char dev[IFNAMSIZ] = {};
@@ -129,7 +131,6 @@ priv_iface_init(int index, char *iface, int proto)
 	must_write(PRIV_UNPRIVILEGED, &index, sizeof(int));
 	strlcpy(dev, iface, IFNAMSIZ);
 	must_write(PRIV_UNPRIVILEGED, dev, IFNAMSIZ);
-	must_write(PRIV_UNPRIVILEGED, &proto, sizeof(int));
 	priv_wait();
 	must_read(PRIV_UNPRIVILEGED, &rc, sizeof(int));
 	if (rc != 0) return -1;
@@ -206,8 +207,10 @@ asroot_ctl_cleanup()
 	int rc = 0;
 
 	must_read(PRIV_PRIVILEGED, &len, sizeof(int));
+	if (len < 0 || len > PATH_MAX)
+		fatalx("privsep", "too large value requested");
 	if ((ctlname = (char*)malloc(len+1)) == NULL)
-		fatal("ctlname", NULL);
+		fatal("privsep", NULL);
 
 	must_read(PRIV_PRIVILEGED, ctlname, len);
 	ctlname[len] = 0;
@@ -251,15 +254,13 @@ asroot_iface_init()
 {
 	int rc = -1, fd = -1;
 	int ifindex;
-	int proto;
 	char name[IFNAMSIZ];
 	must_read(PRIV_PRIVILEGED, &ifindex, sizeof(ifindex));
 	must_read(PRIV_PRIVILEGED, &name, sizeof(name));
 	name[sizeof(name) - 1] = '\0';
-	must_read(PRIV_PRIVILEGED, &proto, sizeof(proto));
 
 	TRACE(LLDPD_PRIV_INTERFACE_INIT(name));
-	rc = asroot_iface_init_os(ifindex, name, &fd, proto);
+	rc = asroot_iface_init_os(ifindex, name, &fd);
 	must_write(PRIV_PRIVILEGED, &rc, sizeof(rc));
 	if (rc == 0 && fd >=0) send_fd(PRIV_PRIVILEGED, fd);
 	if (fd >= 0) close(fd);
@@ -313,8 +314,10 @@ asroot_iface_description()
 	must_read(PRIV_PRIVILEGED, &name, sizeof(name));
 	name[sizeof(name) - 1] = '\0';
 	must_read(PRIV_PRIVILEGED, &len, sizeof(int));
+	if (len < 0 || len > PATH_MAX)
+		fatalx("privsep", "too large value requested");
 	if ((description = (char*)malloc(len+1)) == NULL)
-		fatal("description", NULL);
+		fatal("privsep", NULL);
 
 	must_read(PRIV_PRIVILEGED, description, len);
 	description[len] = 0;
@@ -344,6 +347,7 @@ asroot_snmp_socket()
 
 	if (!addr) {
 		addr = (struct sockaddr_un *)malloc(sizeof(struct sockaddr_un));
+		if (!addr) fatal("privsep", NULL);
 		must_read(PRIV_PRIVILEGED, addr, sizeof(struct sockaddr_un));
 	} else
 		/* We have already been asked to connect to a socket. We will
@@ -517,8 +521,8 @@ sig_chld(int sig)
 /* Create a directory recursively. */
 static int mkdir_p(const char *pathname, mode_t mode)
 {
-	char path[PATH_MAX+1], current[PATH_MAX+1] = {};
-	char *tok;
+	char path[PATH_MAX+1];
+	char *current;
 
 	if (strlcpy(path, pathname, sizeof(path)) >= sizeof(path)) {
 		errno = ENAMETOOLONG;
@@ -526,14 +530,12 @@ static int mkdir_p(const char *pathname, mode_t mode)
 	}
 
 	/* Use strtok which will provides non-empty tokens only. */
-	if (path[0] == '/') current[0] = '/';
-	tok = strtok(path, "/");
-	while (tok) {
-		strcat(current, tok);
-		if (mkdir(current, mode) != 0 && errno != EEXIST)
+	for (current = path + 1; *current; current++) {
+		if (*current != '/') continue;
+		*current = '\0';
+		if (mkdir(path, mode) != 0 && errno != EEXIST)
 			return -1;
-		strcat(current, "/");
-		tok = strtok(NULL, "/");
+		*current = '/';
 	}
 
 	errno = 0;
@@ -685,7 +687,11 @@ priv_caps(uid_t uid, gid_t gid)
 }
 
 void
+#ifdef ENABLE_PRIVSEP
 priv_init(const char *chrootdir, int ctl, uid_t uid, gid_t gid)
+#else
+priv_init(void)
+#endif
 {
 
 	int pair[2];
